/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
*    This file is the entry point of the firestorm sensor;
*    it is responsible for initialising all other components.
*    It is also where the plugin import callback routine lives.
*/
#include <config.h>
#include <time.h>
#include <signal.h>
#include <string.h>
#include <config.h>
#include <conf.h>
#include <packet.h>
#include <signature.h>
#include <realm.h>
#include <decode.h>
#include <capdev.h>
#include <capture.h>
#include <cleanup.h>
#include <loader.h>
#include <parser.h>
#include <preproc.h>
#include <matcher.h>
#include <template.h>
#include <alert.h>
#include <target.h>
#include <alert.h>
#include <sensor.h>

sig_atomic_t caught_hup=0;

const char default_config[]="/etc/firestorm.conf";

char *the_config=(char *)default_config;

/* Initialised forwards, cleaned up backwards */
proc_subsys_init subsys_init[]={
	capdev_init,
	parser_init,
	preproc_init,
	decode_init,
	realm_init,
	alert_init,
	capture_init,
	NULL
};

/* Resources to export to plugins */
struct {
	const char *name;
	void *data;
}exports[]={
	{"realm.add",		realm_add},
	{"realm.add2",		realm_add2},
	{"realm.find",		realm_find},
	{"capdev.find", 	capdev_find},
	{"capdev.add", 		capdev_add},
	{"decode.add", 		decode_add},
	{"decode.subproto", 	decode_subproto},
	{"decode.proto", 	decode_proto},
	{"decode.__pcap_dlt", 	&pcap_dlt_p},
	{"parser.add", 		parser_add},
	{"parser.load", 	parser_load},
	{"target.add", 		target_add},
	{"target.find",		target_find},
	{"capture.add", 	capture_add},
	{"preproc.add", 	preproc_add},
	{"preproc.find",	preproc_find},
	{"preproc.dispatch", 	preproc_dispatch},
	{"preproc.activate", 	preproc_activate},
	{"signature.rule",	signature_rule},
	{"signature.abort", 	signature_abort},
	{"signature.criteria",	signature_criteria},
	{"signature.modifier",	signature_modifier},
	{"signature.alert", 	signature_alert},
	{"signature.commit",	signature_commit},
	{"matcher.add",		matcher_add},
	{"matcher.find",	matcher_find},
	{"template.ptr",	template_ptr},
	{"template.ptrptr",	template_ptrptr},
	{"template.longrange",	template_longrange},
	{"template.shortrange",	template_shortrange},
	{"alert",		alert},
	{"alert.generator",	alert_generator},
	{"serial_number",	serial_number},
	{NULL, NULL}
};

void *plugin_callback(const char *item)
{
	u_int32_t i;

	for(i=0; exports[i].name; i++) {
		if ( !strcmp(exports[i].name, item) ) {
			return exports[i].data;
		}
	}

	printf("resource: request for unknown object: %s\n", item);
	return NULL;
}

void deadly_signal(int signum, siginfo_t *info, void *foo)
{
	printf("signal: Got signal %u from PID=%u, cleaning up\n",
		signum, info->si_pid);

	capture_stop_all();
}

void hup_signal(int signum, siginfo_t *info, void *foo)
{
	printf("signal: Got signal %u from PID=%u, rehupping\n",
		signum, info->si_pid);

	caught_hup=1;
}

int main ( int argc, char **argv ) 
{
	struct sigaction sig;
	u_int32_t i;
	
	printf("Firestorm v" VERSION "\n"
		"Copyright (c) 2002 Gianni Tedesco\n"
		"Released under the terms of the "
		"GNU GPL version 2 (see: COPYING)\n\n");
	
#ifdef HAVE_TZSET
	/* Set timezone from locale */
	tzset();
#endif

	/* Initialise our subsystems */
	for(i=0; subsys_init[i]; i++)
		subsys_init[i]();

	/* Initialise plugin subsystem */
	loader_init(plugin_callback);

	if ( argc > 1 )
		the_config=argv[1];

	/* Read in the config file */
	conf_load(the_config);

	/* Apply all configs */
	conf_apply();

	/* Catch INT and TERM for clean exits */
	sig.sa_sigaction=deadly_signal;
	sig.sa_flags=SA_SIGINFO;
	sigemptyset(&sig.sa_mask);
	sigaddset(&sig.sa_mask, SIGINT);
	sigaddset(&sig.sa_mask, SIGTERM);
	sigaction(SIGINT, &sig, NULL);
	sigaction(SIGTERM, &sig, NULL);

	/* Catch HUP to rotate logs */
	sig.sa_sigaction=hup_signal;
	sig.sa_flags=SA_SIGINFO;
	sigemptyset(&sig.sa_mask);
	sigaction(SIGHUP, &sig, NULL);

	/* This runs the first capture to completion */
	capture_go();
	
	cleanup(EXIT_OK, "Firestorm exiting normally");
}
